#include "alabel.h"

#include <qpainter.h>
#include <qfont.h>
#include <qrect.h>

ALabel::ALabel(QWidget *parent, bool notif)
		:QWidget(parent, "Label", WStyle_Customize | WType_Popup | 
			 WStyle_Tool | WStyle_StaysOnTop),
         notif_type(notif) 
{
        bfont.setBold(true);
        bfont.setPointSize(14);
        bfont.setStyleStrategy(QFont::PreferAntialias);

        nfont.setPointSize(10);
        nfont.setStyleStrategy(QFont::PreferAntialias);
	warn_draw = false;
}

void ALabel::setText(const char* s)
{
		text = s;
        update();
}

void ALabel::setTitleMessage(const QString & s, const QString & title)
{
		warn_draw = true;
		warning = s;
		this->title = title;
        update();
}

void ALabel::stopWarning()
{
		warn_draw = false;
}

QString ALabel::getText() const { return text; }

void ALabel::paintEvent(QPaintEvent *)
{
        QPainter p(this);
	QRect r = rect();
        
	p.setFont(nfont);
	if(notif_type) {
                r.setTop(r.top() + 15);
                r.setLeft(r.left() + 30);
                if(text.length() < 30 && !warn_draw) 
                        r.setLeft(r.left() + 20);
        } else {
                r.setTop(r.top() + 4);
                r.setLeft(r.left() + 12);
		}
		if(!warn_draw && notif_type)
                r.setTop(r.top() + 15);
        
		p.drawText(r, Qt::AlignLeft | Qt::DontClip, text);
		if(warn_draw) {
                r.setTop(r.top() + 15);
                // push to stack
                p.save();
                p.setFont(bfont);
                p.drawText(r, Qt::AlignLeft | Qt::DontClip, title);
		
                r.setTop(r.top() + 25);
                // restore from stack
                p.restore();
                p.drawText(r, Qt::AlignLeft | Qt::DontClip, warning);
		}
}

void ALabel::mouseReleaseEvent(QMouseEvent*)
{
        if(notif_type)
                hide();
}
