/***************************************************************************
                          opkdekiosk.cpp  -  description
                             -------------------
    begin                : Mon Jun 20 10:42:25 PHT 2005
    copyright            : (C) 2005 by Abdiel Janulgue
    email                : xynopsis@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <kglobal.h>
#include <klocale.h>
#include <kconfig.h>
#include <kapp.h>
#include <kmessagebox.h>
#include <kstandarddirs.h>
#include <kiconloader.h>
#include <kprocess.h>
#include <dcopref.h>
#include <kaboutdata.h>
#include <kaboutapplication.h>

#include <qlayout.h>
#include <qlabel.h>
#include <qcolor.h>
#include <qfont.h>
#include <qpopupmenu.h>
#include <qcursor.h>
#include <qtimer.h>
#include <qpixmap.h>
#include <qevent.h>
#include <qtooltip.h>

#include "opkdekiosk.h"

extern "C"
{
  KPanelApplet* init( QWidget *parent, const QString configFile)
  {
    KGlobal::locale()->insertCatalogue("opkdekiosk");
    return new OpKDEkiosk(configFile, KPanelApplet::Normal,
                      KPanelApplet::About | KPanelApplet::Preferences,
                      parent, "opkdekiosk");
  }
}

OpKDEkiosk::OpKDEkiosk(const QString& configFile, Type type, int actions,
                       QWidget *parent, const char *name) 
        :KPanelApplet(configFile, type, actions, parent, name)
{
        ksConfig = config();
        ab = 0;
        pop = 0;
        
        QHBoxLayout* vl= new QHBoxLayout(this, 6, 6);
        
        timerLabel = new QLabel(this);
        timerLabel->setFrameStyle( QFrame::Box | QFrame::Sunken );
        timerLabel->setPaletteBackgroundPixmap(QPixmap(::locate("data",
                                                      "openkiosk/metal.png")));
        QFont def;        
        def.setBold(true);
        def.setPointSize(14);
        timerLabel->setFont(def);
        timerLabel->setAlignment( Qt::AlignHCenter | Qt::AlignVCenter);
        timerLabel->setText("0:00");
        fb = new FlatButton(this);
	connect(fb, SIGNAL(clicked()), SLOT(updateLogoff()) );
	fb->setUsesTextLabel ( true );
	fb->setTextPosition ( QToolButton::BesideIcon );
        fb->setIconSet(::SmallIcon("exit"));
        vl->add(fb);
        vl->add(timerLabel);

        initLock();
}

OpKDEkiosk::~OpKDEkiosk()
{	
}

void OpKDEkiosk::initLock()
{
        pop = new QPopupMenu(this);
        pop->insertItem(::SmallIcon("clock"),
                        tr2i18n("&Check Remaining Time"), this,
                        SLOT(checkTime()) );
    	pop->insertSeparator ();
        pop->insertItem(::SmallIcon("exit"),
                        tr2i18n("&Log off or pause usage"), this,
                        SLOT(updateLogoff()));
        pop->insertItem(::SmallIcon("reload"),
                        tr2i18n("&Reload/extend account"), this,
                        SLOT(reload()));
        pop->insertItem(::SmallIcon("password"),
                        tr2i18n("Change my &Password"), this,
                        SLOT(changePassword()));
        pop->insertSeparator();
        pop->insertItem(::SmallIcon("configure"),
                        tr2i18n("Confi&gure"), this,
                        SLOT(preferences()));
        pop->insertItem(tr2i18n("&About"), this,
                        SLOT(about()));
        fb->pop = pop;
        
        KConfig kioskc(::locate("config", "kioskclientrc"));
        kioskc.setGroup("kioskclient");
        host = kioskc.readEntry("host", "none");
	port = kioskc.readEntry("port","10018").toInt();
	localname = kioskc.readEntry("station");
        
	procCheck = new QTimer(this, "procCheck");
        if(host == "none") {
                preferences();
                return;
        } else {
		KProcess p;
		p << "kiosklock" 
		  << "--server" << host 
		  << "--port" << QString::number(port)
		  << "--localname" << localname;
		p.start(KProcess::DontCare);
		kioskpid = p.pid();
        }
	connect(procCheck, SIGNAL(timeout()), SLOT(checkProcess()) );
	procCheck->start(4000);
}

void OpKDEkiosk::checkProcess()
{
	QString procname = QString("kiosklock-") + QString::number(kioskpid);
	DCOPRef kd(procname.ascii(), "KioskLock");

	if(!kd.isNull()) {
		
		DCOPReply d = kd.call("timerDisplay");
		if(d.isValid()) {
			QString timerd = d;
			timerLabel->setText(timerd);
		} else {
			KProcess p;
			p << "kiosklock" 
			  << "--server" << host 
			  << "--port" << QString::number(port)
			  << "--localname" << localname;
			p.start(KProcess::DontCare);
			kioskpid = p.pid();
		}		
	}
}

void OpKDEkiosk::about()
{
	if(!ab) {
                ab = new KAboutData("opkdekiosk", I18N_NOOP("OpenKDEKiosk"), 
                                    "2.0.4", I18N_NOOP("The KDE client of "
						       "OpenKiosk for "
						       "UNIX, Linux, and "
						       "BSD on X11"),
                                    KAboutData::License_GPL, 
				    "(c) 2005 by xynopsis",
				    I18N_NOOP("Hinimo sa pinoy!"),
                                    "http://openkiosk.sourceforge.net");
                ab->addAuthor("Abdiel B. Janulgue",
                              "http://openkiosk.sourceforge.net",
                              "xynopsis@users.sourceforge.net");
        }
	
        KAboutApplication dialog(ab);
        dialog.exec();
}

void OpKDEkiosk::preferences()
{
        KProcess p;
        p << "kdesu";
        p << "kiosksetup";
        p.start(KProcess::DontCare);
}

void OpKDEkiosk::checkTime()
{
	QString procname = QString("kiosklock-") + QString::number(kioskpid);
	DCOPRef kproc(procname.ascii(), "KioskLock");
	
	if(!kproc.isNull()) 
		kproc.call("checkTime");
}

void OpKDEkiosk::updateLogoff()
{
	QString procname = QString("kiosklock-") + QString::number(kioskpid);
	DCOPRef kproc(procname.ascii(), "KioskLock");
	
	if(!kproc.isNull()) 
		kproc.call("updateLogoff");
}

void OpKDEkiosk::reload()
{
	QString procname = QString("kiosklock-") + QString::number(kioskpid);
	DCOPRef kproc(procname.ascii(), "KioskLock");
	
	if(!kproc.isNull()) 
		kproc.call("reload");

}

void OpKDEkiosk::changePassword()
{
	QString procname = QString("kiosklock-") + QString::number(kioskpid);
	DCOPRef kproc(procname.ascii(), "KioskLock");
	
	if(!kproc.isNull()) 
		kproc.call("changePassword");

}

int OpKDEkiosk::widthForHeight( int height ) const
{
        return height + 200;
}

int OpKDEkiosk::heightForWidth( int width) const
{
        return width;
}

// void OpKDEkiosk::resizeEvent( QResizeEvent *e)
// {
//         KPanelApplet::resizeEvent(e);
//        //  fb->setMinimumSize( QSize(width(), 20) );
// //         if(height() < 40)
// //                 fb->hide();
// //         else
// //                 fb->show();
// }

void OpKDEkiosk::mousePressEvent(QMouseEvent*e)
{
        if(pop && (e->button () == RightButton))
                pop->exec(QCursor::pos());
        else
		checkTime();
}

FlatButton::FlatButton(QWidget * parent, const char * name)
        :QToolButton(parent, name)
{
        pop = 0;
	setTextLabel(tr2i18n("Logoff Kiosk")); 
        QToolTip::add(this, tr2i18n("Logoff or pause session"));
}
       
void FlatButton::mousePressEvent(QMouseEvent*e)
{       
	if(pop && (e->button () == RightButton)) {
                pop->exec(QCursor::pos());
                return;
        }	
	QToolButton::mousePressEvent(e);
	       
}
