// appselector.cpp: implementation of the AppSelector class.
//
//////////////////////////////////////////////////////////////////////

#include "appselector.h"
#include <qtimer.h>
#include <qpainter.h>
#include <sstream>
#include <qcolor.h>
#include <qapplication.h>
#include <qdesktopwidget.h>
#include <qprocess.h>
#include <qmessagebox.h>

#include <kstandarddirs.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <X11/Xatom.h>
#include <X11/Intrinsic.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

AButton::AButton(QWidget* parent,const char* text, bool istext)
        :QWidget(parent,"AButton"), b_text(text)
{
        is_text = istext;
        if(!is_text) {
                bg = QPixmap(locate("data", QString("openkiosk/")+text));
                resize(16, 16);
        } 
}

void AButton::setText(const char*text)
{
        b_text = text;
        update();
}

void AButton::paintEvent(QPaintEvent *)
{
        QPainter p(this);
        
        if(!is_text)
                p.drawPixmap(0,0, bg);
        else 
                //p.drawText((width() / 2) - 10,20, b_text); 
                p.drawText( rect(), AlignCenter, b_text );
}

void AButton::mouseReleaseEvent(QMouseEvent *)
{
        clicked();
}

SelectorItem::SelectorItem(QWidget* parent, const char* name, const char* img, 
                           bool buttonType)
        :QWidget(parent, "SelectorItem"), 
         text(name), selected(false), isButton(buttonType), isClicked(false)
{
        setBackgroundColor(QColor(155, 145, 251));
        initbg = QColor(155, 145, 251);
        sel = QPixmap(locate("data", QString("openkiosk/")+img));
        resize(180, sel.height());
}

void SelectorItem::setText(const char* msg)
{
        text = msg;
        update();
}

void SelectorItem::paintEvent(QPaintEvent *)
{
        QPainter p(this);
        
        int x = 0;
        int y = 0;
        if(isButton && isClicked) {
                x = 1;
                y = 1;
                isClicked = false;
        }	
        p.drawPixmap(x,y, sel);

        int top = (sel.height() / 2) ;
        int left = sel.width() + 5;
        top += y;
        left += x;
        p.drawText(left, top, text);
}

void SelectorItem::mouseReleaseEvent(QMouseEvent *)
{
        if(isButton)
                update();
        emit clicked(text);
}

void SelectorItem::enterEvent(QEvent*)
{
        selected = true;
        setBackgroundColor(QColor(49, 106, 197));
        update();
}

void SelectorItem::leaveEvent(QEvent*)
{
        selected = false;
        setBackgroundColor(initbg);	
        update();
}

void SelectorItem::mousePressEvent(QMouseEvent*)
{
        if(isButton) {
				isClicked = true;
				update();
        }
}

	

AppSelectorItem::AppSelectorItem(AppSelector* parent, const char* name,
                                 const char* path)
        :SelectorItem(parent->econtainer, name, "app_icon.png"),
         app_path(path), appsel(parent)
{
        setBackgroundColor(QColor(230, 232, 245));
        initbg = QColor(230, 232, 245);
        move(3,parent->heightC);
        parent->heightC += sel.height();
        
        // grow
        if(parent->heightC > max_cont_len)
                parent->econtainer->resize(parent->viewport->width(),
                                           parent->heightC);
        connect(this, SIGNAL(clicked(const QString&)),
                SLOT(exec(const QString&)));
        parent->items.push_back(this);
}

void AppSelectorItem::exec(const QString &)
{	
        execprog(app_path);
        appsel->minimize();
}

void AppSelectorItem::kill()
{
        for(std::vector<QProcess*>::iterator it = processes.begin(); it != processes.end();
            ++it) {
                (*it)->tryTerminate();
                QTimer::singleShot( 5000, (*it), SLOT( kill() ) );
        }
}

void AppSelectorItem::execprog(const char* a)
{
        QProcess *p = new QProcess(this);
        p->addArgument(a);
        
      	QString warn = tr("Cannot execute ");
        warn += a;
        warn += tr(".\nPlease report this to the administrator");
        
        // Start the child process
        if(!p->start())
        {
                QMessageBox::information(0, tr("Process status"), warn);
                return;
        }
        processes.push_back(p);
}

AppSelector::AppSelector(QWidget *parent)
        :QWidget(parent, "OpenKiosk Panel", WStyle_Customize |  
		 WType_TopLevel | WX11BypassWM), 
         heightC(0), movY(0)
{	
        int xlen = 200;
        int ylen = 470;
        mintm = new QTimer(this);
        restm = new QTimer(this);
        connect(mintm, SIGNAL(timeout()), SLOT(minAnimate()) );
        connect(restm, SIGNAL(timeout()), SLOT(resAnimate()) );
        
        setBackgroundColor(QColor(155, 145, 251));
        restb = new AButton(this, "restore.png");
        restb->setPaletteBackgroundColor(QColor(155, 145, 251));
        connect(restb, SIGNAL(clicked()) , SLOT(restore()) );
        restb->resize(64,10);
        restb->move(100 - 32, -100);
        
/*	mydocs = new SelectorItem(this, "My Documents",
    "mydocs_sel.bmp", "mydocs_unsel.bmp");
	mydocs->move(10, 15);
	mydocs->resize(xlen - 30, 22);
	mydocs->clicked.connect(SLOT(this, AppSelector::openDocs));
    
	AButton *li = new AButton(this, "splitter.bmp");
	li->resize(200,10);
	li->move(0, 40); */
        header = new AButton(this, "header.png");
        header->resize(200, 43);
        header->move(0,0);
        
        viewport = new QWidget(this, "viewport");
        viewport->resize(xlen - 20, max_cont_len);
        viewport->move(10,60);
        econtainer = new QWidget(viewport, "EntryContainer");
        econtainer->move(0,0);
        econtainer->resize(xlen - 20, max_cont_len);
        econtainer->setBackgroundColor(QColor(230, 232, 245));
        
        AButton *upb = new AButton(this, "up.png");
        upb->setPaletteBackgroundColor(QColor(155, 145, 251));
        connect(upb, SIGNAL(clicked()), SLOT(scrollUp()) );
        upb->move(140, max_cont_len + 65);
        AButton *dwb = new AButton(this, "down.png");
        dwb->setPaletteBackgroundColor(QColor(155, 145, 251));
        connect(dwb, SIGNAL(clicked()), SLOT(scrollDown()) );
        dwb->move(164, max_cont_len + 65);
        
        AButton *line = new AButton(this, "line.png");
        line->setPaletteBackgroundColor(QColor(155, 145, 251));
        line->resize(200,10);
        line->move(0, max_cont_len + 80);
    
        timedisp = new AButton(this, "00:00", true);
        timedisp->setBackgroundColor(QColor(230, 232, 245));
        timedisp->resize(xlen - 20, 30);
        timedisp->move(10, max_cont_len+ 95);
        
        AButton *line2 = new AButton(this, "line.png");
        line2->setPaletteBackgroundColor(QColor(155, 145, 251));
        line2->resize(200,10);
        line2->move(0, max_cont_len + 130);
        
        SelectorItem *logoff = new SelectorItem(this, tr("Logoff"),
                                                "exit.png", true);
        connect(logoff, SIGNAL(clicked(const QString&)),
                SLOT(logoffed(const QString&)) );
        logoff->move(50, ylen - 50);
        logoff->resize(100, 32);
        
        minb = new AButton(this, "minimize.png");
        minb->setPaletteBackgroundColor(QColor(155, 145, 251));
        connect(minb, SIGNAL(clicked()), SLOT(minimize()) );
        minb->resize(64,10);
        minb->move(100 - 32, ylen - 10); 
        resize(xlen, ylen);
        
        QDesktopWidget *d = QApplication::desktop();    
        screen_height = d->height();
        ypos = (screen_height / 2) - (ylen / 2);
        init_ypos = ypos;
        mov_int = (screen_height - ypos) / 30;
        move(0, ypos);
}

AppSelector::~AppSelector()
{
        
}
/*
  LRESULT WINAPI AppSelector::messageHandler(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
  {
  POINT pt;
  std::ostringstream dbg;
  
  switch(msg) {
  case WM_CLOSE:
  // send to eternity
			return 0;
            default:
			return DefWindowProc(hwnd, msg, wparam, lparam);
	}
	return 0;
    }
*/
void AppSelector::clear()
{
        for(ItemList::iterator it = items.begin(); it != items.end(); ++it)
                delete (*it);
        
        items.clear();
        
        heightC = 0;
        movY = 0;
}

void AppSelector::killApps()
{
        for(ItemList::iterator it = items.begin(); it != items.end(); ++it)
                (*it)->kill();
}

void AppSelector::setTimeDisplay(const char* time)
{
        timedisp->setText(time);
}

void AppSelector::scrollDown()
{
        if((movY == - (heightC - max_cont_len)) ||
           heightC < max_cont_len)
                return;
        movY -= 20;
        econtainer->move(0, movY);
}

void AppSelector::scrollUp()
{	
        if(movY == 0)
                return;
        movY += 20;
        econtainer->move(0, movY);
}

void AppSelector::minimize()
{
        setEnabled(false);
        mintm->start(2);
}

void AppSelector::restore()
{
        setEnabled(false);
        restm->start(2);
}

void AppSelector::minAnimate()
{
        if((ypos < (screen_height - 45)) && 
           (ypos > (screen_height - 60)) ) {
                ypos = screen_height - 60;
                move(0, screen_height - 60);
                header->move(0,-100);
                restb->move(100 - 32, 1);
                mintm->stop();
                setEnabled(true);

                // Window placement
                Window wins[2];
                wins[0] = taskbar->winId();
                wins[1] = winId();
                XRestackWindows(qt_xdisplay(), wins, 2);

                update();
                return;
        }
        ypos += mov_int;
        move(0, ypos);
}

void AppSelector::resAnimate()
{
        if((ypos > (init_ypos - 15 )) && 
           (ypos < (init_ypos)) ) {
                ypos = init_ypos;
                move(0, init_ypos);
                restb->move(100 - 32, -100);
                header->move(0,0);
                restm->stop();
                setEnabled(true);
                update();
                return;
        }
        ypos -= mov_int;
        move(0, ypos);
}

void AppSelector::logoffed(const QString&)
{
        emit logOff();
}
