// appselector.h: interface for the AppSelector class.
//
//////////////////////////////////////////////////////////////////////

#ifndef _APPSELECTOR_H_
#define _APPSELECTOR_H_

#include <qwidget.h>
#include <list>
#include <vector>
#include <qpixmap.h>
#include <qcolor.h>

const int max_cont_len = 280;

class AppSelectorItem;
class QTimer;
class QProcess;

class AButton: public QWidget
{
        Q_OBJECT
public:
        AButton(QWidget* parent,const char* text, bool istext = false);
        void setText(const char*text);

signals:
        void clicked();

protected:
        void paintEvent(QPaintEvent *);
        void mouseReleaseEvent(QMouseEvent *);
        
private:
        QString b_text;
        QPixmap bg;
        bool is_text;
};

class SelectorItem: public QWidget
{
        Q_OBJECT        
 public:
        SelectorItem(QWidget* parent, const char* name,
                     const char* img, bool buttonType = false);        
        void setText(const char* msg);

 signals:
        void clicked(const QString&);
        
 protected:
        void paintEvent(QPaintEvent *);
        void mouseReleaseEvent(QMouseEvent *);
        void mousePressEvent(QMouseEvent*);
        void enterEvent(QEvent*);
        void leaveEvent(QEvent*);
        
        QPixmap sel;
        QColor initbg;
        
 private:
        QString text;
        bool selected;
        bool isButton;
        bool isClicked;
};

class AppSelector : public QWidget  
{
        Q_OBJECT
        friend class AppSelectorItem;
 public:
        typedef std::list<AppSelectorItem*> ItemList;
	
        AppSelector(QWidget* parent);
        virtual ~AppSelector();
        void setTimeDisplay(const char*);
        void killApps();
        void clear();
        void setTaskbarWidget(QWidget* t) { taskbar = t; }

 signals:        
        void logOff();
        
 private slots:
        void scrollDown();
        void scrollUp();
        void minimize();
        void restore();
        void minAnimate();
        void resAnimate();
        void logoffed(const QString&);

 private:    
        int heightC;
        QWidget *viewport;
        QWidget *econtainer;
        QWidget *taskbar;
        AButton *restb;
        AButton *minb;
        AButton *timedisp;
        AButton *header;
        QTimer *mintm;
        QTimer *restm;
        int movY;
        int ypos;
        int init_ypos;
        int mov_int;
        int screen_height;
        ItemList items;
};

class AppSelectorItem: public SelectorItem
{
        Q_OBJECT        
 public:
        AppSelectorItem(AppSelector* parent, const char* name, const char* path);
        void execprog(const char* a);
        void kill();

 private slots:
        void exec(const QString &);
        
 private:
        QString app_path;
        AppSelector* appsel;
        std::vector<QProcess*> processes;
};

#endif // _APPSELECTOR_H_
