// changepassdlg.cpp: implementation of the ChangePassDlg class.
//
//////////////////////////////////////////////////////////////////////

#include "changepassdlg.h"
#include "kiosklock.h"

#include <qapplication.h>
#include <qrect.h>
#include <qmessagebox.h>
#include <qlineedit.h>
#include <qpushbutton.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ChangePassDlg::ChangePassDlg(KioskLock *lock)
        :ChangePass(lock, "ChangePass"), lockinst(lock), isNewAccount(false)
{
	isOpen = false;
    connect(ok, SIGNAL(clicked()), SLOT(hide()) );
    connect(cancel, SIGNAL(clicked()), SLOT(hide()));
}

void ChangePassDlg::show()
{
        isNewAccount ? oldpass->setEnabled(false) : oldpass->setEnabled(true);
        
        QDesktopWidget *d = QApplication::desktop();   
        int x = d->width();
        int y = d->height();
        move((x / 2) - (rect().right() / 2), 
             (y / 2) - (rect().bottom() /2)
             );
        isOpen = true;
        QDialog::show();
}

void ChangePassDlg::hide()
{        
        isOpen = false;
        if(sender() == cancel) {
                QDialog::hide();
                return;
        }
        
        QString oldp = oldpass->text();
        QString newp = newpass->text();
        QString conp = confpass->text();
        
        int passlen = newp.length();
        if(!(newp == conp)) {
                QMessageBox::warning(this, tr("Error"), tr("The passwords you type do not match"),
                                     QMessageBox::Ok, QMessageBox::NoButton);
				return;
        } else if( newp.isEmpty() ) { 
                QMessageBox::warning(this, tr("Error"), tr("Empty passwords not allowed"),
                                     QMessageBox::Ok, QMessageBox::NoButton);
				return;
        } else if(!( (passlen >= minpasslen) && (passlen <= maxpasslen) )) {
				QString err = tr( "Password length should be greater than or equal to %1"
                                  "\nand should be less than or equal to %2")
                        .arg(minpasslen)
                        .arg(maxpasslen);
                QMessageBox::warning(this, tr("Error"), err,
                                     QMessageBox::Ok, QMessageBox::NoButton);
				return;
        } 
        XClient c(lockinst, lockinst->serv_ip, lockinst->serv_port);
        XmlRpcValue args;
        XmlRpcValue result;
        args["name"] = lockinst->serial;
        args["key"] = oldp;
        args["newkey"] = newp;
        args["station"] = lockinst->_station;
        std::string names = args["name"];
        bool status = false;
        if (c.execute("opkiosk.changepass", args, result)) 
				status  = (bool) result["status"] ;
        if(status) {
				lockinst->key = newp;
				QMessageBox::information(this, tr("Success"), tr("Password successfully updated"),
                                         QMessageBox::Ok);
				QDialog::hide();
        }			
        newpass->setText("");
        oldpass->setText("");
        confpass->setText("");
}

void ChangePassDlg::setPasswordLength(int min, int max)
{
        minpasslen = min;
        maxpasslen = max;
}

void ChangePassDlg::setNewAccount(bool a)
{
        isNewAccount = a;
}

ChangePassDlg::~ChangePassDlg()
{
}
