/***************************************************************************
 *   Copyright (C) 2004 by Abdiel Janulgue                                 *
 *   xynopsis at users dot sourceforge dot net                             *
 ***************************************************************************/
//
// X11 screen locking mechanism code portion:
// Copyright (c) 1999 Martin R. Jones <mjones@kde.org>
// Copyright (c) 2003 Oswald Buddenhagen <ossi@kde.org>
//
// OpenKiosk Win32 client - Win32Lock
// kiosklock.cpp: implementation of the KioskLock class.
//
//////////////////////////////////////////////////////////////////////
#include "kiosklock.h"

#include <string>
#include <qpainter.h>
#include <qapplication.h>
#include <qcolor.h>
#define RGB(r,g,b) QColor(r,g,b)

#include <qlineedit.h>
#include <qmessagebox.h>
#include <qstringlist.h>
#include <qfile.h>
#include <qdatastream.h>
#include <qcursor.h>
#include <qdialog.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qframe.h>
#include <qapplication.h>

#include <kstandarddirs.h>
#include <kmessagebox.h>
#include <kapplication.h>
#include <kconfig.h>
#include <dcopclient.h>
#include <dcopobject.h>

#include "appselector.h"
#include "serialdialog.h"
#include "changepassdlg.h"
#include "logout.h"
//#include "ListenerServ.h"

#include <unistd.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <X11/Xatom.h>
#include <X11/Intrinsic.h>

ExtendDlg:: ExtendDlg(KioskLock* kl)
        :Extend(kl, "ExtendDlg"), lockinst(kl)
{
        isOpen = false;
}

bool ExtendDlg::active() const
{
        return isOpen;
}

int ExtendDlg::exec()
{
        QDesktopWidget *d = QApplication::desktop();
        
        int x = d->width();
        int y = d->height();
        move((x / 2) - (rect().right() / 2),
             (y / 2) - (rect().bottom() /2)
             );
        netedit->setText("");
        isOpen = true;
        return QDialog::exec();
}

void ExtendDlg::accept()
{
        lockinst->update(lockinst->seconds / 60, netedit->text());
        netedit->setText("");
        QDialog::accept();
        isOpen = false;
}

void ExtendDlg::reject()
{
        isOpen = false;
        QDialog::reject();
}

InfoLabel::InfoLabel(QWidget *parent)
        :ALabel(parent, true)
{
        connect(&animt, SIGNAL(timeout()), SLOT(hidewindow()) );
}

void InfoLabel::setInfoText(const char* text, int duration)
{
        setText(formatText(text).c_str());
        show();
        
        if(duration > 0)
                animt.start(duration * 1000, true);
}

void InfoLabel::setWarning(const QString& message, const char *title)
{
        std::string smsg = message.ascii();
        setTitleMessage(formatText(smsg).c_str(), title);
}

// Try to fit s inside this label with width wid,
std::string InfoLabel::formatText(std::string s, int wid)
{
        int npos = 0;
        int len = s.length();
        
        if( len > wid ) {
                if(s.substr(wid,1) != " ") {
                        npos = s.rfind(" ", wid);
                } else
                        npos = wid;
                
                s.insert(npos+1, "\n");
                std::string subs = s.substr(npos, len - npos);
                s.erase(npos, len-npos);
                s.insert(npos, formatText(subs));
                return s;
        } else
                return s;
}

// TODO: animate hiding like MSN/Yahoo Messenger notification
void InfoLabel::hidewindow()
{
        hide();
        stopWarning();
}

TimerLabel::TimerLabel(QWidget *parent)
        :ALabel(parent)
{
        connect(&flasher, SIGNAL(timeout()), SLOT(flash()) );
        connect(&flasher_i, SIGNAL(timeout()), SLOT(off() ));
        connect(&oftimer, SIGNAL(timeout()), SLOT(startoff()) );
        timer_started = false;
}

void TimerLabel::startFlasher()
{
        flasher.start(1 * 1000);
        oftimer.start(500, true);
}

void TimerLabel::stopFlasher()
{
        flasher.stop();
        flasher_i.stop();
}

void TimerLabel::flash()
{
        setBackgroundColor(QColor(255,128,0));
        repaint();
}

void TimerLabel::startoff()
{
        flasher_i.start(1 * 1000);
}

void TimerLabel::off()
{
        setBackgroundColor(QColor(255,255,217));
        update();
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
KioskLock::KioskLock(QWidget* aparent, const char * name)
        :DCOPObject("KioskLock"),
	 QWidget(aparent, name, WX11BypassWM)
{
	// Register with DCOP
	if ( !kapp->dcopClient()->isRegistered() ) {
		kapp->dcopClient()->registerAs( "kiosklock" );
		kapp->dcopClient()->setDefaultObject( objId() );
	}

	seconds = 0;
	timer_counter = "0:00";

        bmp = locate("data", "openkiosk/background.bmp");
	checkup_interval = 60;
        locked = true;
        client_port = 0;

        connect(&savetm, SIGNAL(timeout()), SLOT(savestate()) );
        connect(&cdowntm, SIGNAL(timeout()), SLOT(countdown()) );
        connect(&updatetm, SIGNAL(timeout()), SLOT(aliveclient()) );
        kapp->installX11EventFilter(this);

        QDesktopWidget *d = QApplication::desktop();
        int x = d->width();
        int y = d->height();

        warning_init = 600;
        warning_final = 120;
        timeout_value = 3600;

        // get user info
        sdlg = new SerialDialog(this, WX11BypassWM);
        connect(this, SIGNAL(connectionProgress(int)), sdlg,
                SLOT(setProgress(int)));
        
        KConfig kioskc(::locate("config", "kioskclientrc"));
        kioskc.setGroup("kioskclient");
        sdlg->setInfoText(kioskc.readEntry("loginlabel"),
                          kioskc.readEntry("passlabel"),
                          tr("Please enter your access number or, "
                             "for members, your username and password "
                             "to login."),
                          kioskc.readBoolEntry("enablepass"));

        // our small timer view
        timerbox = new TimerLabel(this);
        timerbox->setText("00:00");
        timerbox->setBackgroundColor(RGB(255,255,217));
        timerbox->move(x - 65, y - 53);
        timerbox->resize(60, 23);
        timerbox_visible  = false;

        // our small notification box
        infobox = new InfoLabel(this);
        infobox->setBackgroundColor(QColor(255,255,217));
        infobox->move(x - 225, y - 150);
        infobox->resize(220, 120);

        cpdialog = new ChangePassDlg(this);
        extdlg = new ExtendDlg(this);

        panel = new AppSelector(this);
        connect(panel, SIGNAL(logOff()), SLOT(updateLogoff()));

        resize(x, y);
        move(0,0);
}

void KioskLock::checkTime()
{
        remainingcheck(false);
}

void KioskLock::changePassword()
{
        if(!cpdialog->active())
                cpdialog->show();
}

void KioskLock::reload()
{
        if(!extdlg->active())
                extdlg->show();
}

        // TODO. create nice about box
void KioskLock::about()
{}

void KioskLock::paintEvent(QPaintEvent *)
{
        QPainter p(this);

        QDesktopWidget *d = QApplication::desktop();
        int xpos = d->width();
        int ypos = d->height();
        p.drawPixmap(xpos/2 - (bmp.width() /2), ypos/2 - (bmp.height() /2), bmp);
}

////////////////////////////////////////////////////////////////////
// Kiosk Lock Protocol client implementation
int KioskLock::gettime(const char* serial, const char * pass, bool peek)
{
        int ret = 0;
        XClient c(this, serv_ip, serv_port);
        connect(&c, SIGNAL(status(int)), SLOT(connectProgress(int)) );
        XmlRpcValue args;
        XmlRpcValue result;

        args["name"] = serial;
        args["key"] = pass;
        args["station"] = _station.ascii();
        args["peek"] = XmlRpcValue(peek);

        if (c.execute("opkiosk.authenticate", args, result)) {
                if((bool) result["invalid_group"]) {
                        ErrorDialog(this, tr("You are not allowed to use this "
                                             "account in this computer"));
                        return ret;
                }

                is_member = (bool) result["is_member"] ;
                is_emptypassword = (bool) result["is_emptypassword"];
                int pmax = (int) result["maxpasslength"];
                int pmin = (int) result["minpasslength"];
		std::string bg = result["background_url"];
		bgurl = bg.c_str();
                cpdialog->setPasswordLength(pmin, pmax);
                emit memberStatus(is_member);
                ret = (int) result["remain_minutes"];
        } else {
                std::ostringstream debugmsg;
                debugmsg << "Cannot execute XML-RPC method: opkiosk.authenticate\n"
                         << "server: " << serv_ip << " port : " << serv_port;
                ErrorDialog(this, debugmsg.str().c_str());
        }

        return ret;
}

bool KioskLock::invalidate()
{
        bool ret = false;
        XClient c(this, serv_ip, serv_port);
        XmlRpcValue args;
        XmlRpcValue result;

        args["name"] = serial.ascii();
        args["key"] = key.ascii();
        args["station"] = _station.ascii();

        if (c.execute("opkiosk.invalidate", args, result)) {
                ret = (bool) result["status"] ;
        } else {
                std::ostringstream debugmsg;
                debugmsg << "Cannot execute XML-RPC method: opkiosk.invalidate\n"
                << "server: " << serv_ip << " port : " << serv_port;
                ErrorDialog(this, debugmsg.str().c_str());
        }

        return ret;
}

bool KioskLock::update(int minutes, const char* newserial)
{
        bool ret = false;
        XClient c(this, serv_ip, serv_port);
        XmlRpcValue args;
        XmlRpcValue result;

        args["name"] = serial.ascii();
        args["key"] = key.ascii();
        args["newserial"] = newserial;
        args["remain_minutes"] = minutes;
        args["station"] = _station;

        if (c.execute("opkiosk.update", args, result)) {
                ret = (bool) result["status"] ;
                int ntime = (int) result["total_minutes"];

                if(ntime > 0) {
                        seconds = ntime * 60;

                        if(seconds <= warning_init) {
                                timerbox->setBackgroundColor(RGB(255,128,0));
                                timerbox->repaint();

                                if(seconds <= warning_final)
                                        timerbox->startFlasher();
                        } else {
                                timerbox->setBackgroundColor(RGB(255,255,217));
                                timerbox->repaint();
                        }

                        std::ostringstream rem;
                        rem << "You now have " << ntime << " minute(s) in your account";
                        infobox->setInfoText(rem.str().c_str());
                }
        } else {
                std::ostringstream debugmsg;
                debugmsg << "Cannot execute XML-RPC method: opkiosk.update\n"
                << "server: " << serv_ip << " port : " << serv_port;
                ErrorDialog(this, debugmsg.str().c_str());
        }

        return ret;
}

void KioskLock::aliveclient()
{
        XClient c(this, serv_ip, serv_port);
        XmlRpcValue args;
        XmlRpcValue result;

        args["station"] = _station.ascii();
        args["client_port"] = client_port;
        // dont care about results. just connect to server while it can
        c.execute("opkiosk.aliveclient", args, result);
}

bool KioskLock::isNodeviewUp()
{
	XClient c(this, serv_ip, serv_port);
	XmlRpcValue args;
	XmlRpcValue result;

	args["station"] = _station;
        args["client_port"] = client_port;
	
	return	c.execute("opkiosk.aliveclient", args, result);
}

void KioskLock::getApplist()
{
        XClient c(this, serv_ip, serv_port);
        XmlRpcValue args;
        XmlRpcValue result;

        args["station"] = _station.ascii();

        if(c.execute("opkiosk.applist", args, result)) {
                appsize = result.size();

                if(appsize == 0)
                        return;
                panel->clear();

                for(unsigned int i = 0; i <appsize; i++) {
                        std::string app = result[i];
                        QString qsapp(app.c_str());
                        QStringList apl = QStringList::split(";", qsapp);
                        new AppSelectorItem(panel, apl[0], apl[1]);
                }
        } else {
                std::ostringstream debugmsg;
                debugmsg << "Cannot execute XML-RPC method: opkiosk.applist\n"
                << "server: " << serv_ip << " port : " << serv_port;
                ErrorDialog(this, debugmsg.str().c_str());
        }
}

/////////////////////////////////////////////////////////////////////
// Screen Management

void KioskLock::openScreen(unsigned int secs, const char* serial, const char *key)
{
        seconds = secs;
        init_sec = secs;
        this->serial = serial;
        this->key = key;
        this->is_member = is_member;

        if(secs > 0) {
                savestate();
                // Save info to disk every 5 seconds
                savetm.start(5 * 1000);
                // Have a working count down timer
                cdowntm.start(1 * 1000);

                seconds--;
                remainingcheck(false, true);
                remainingcheck(false);

                hide();
        }
}

void KioskLock::closeScreen()
{
        show();

        savetm.stop();
        cdowntm.stop();

        seconds = 0;
        savestate();
        
        emit sessionEnd(kioskmode);
}

void KioskLock::lockinvalidate()
{
        if(locked)
                return;

        // Check if extended. Peek at account only
        int mins = gettime(serial,key, true);
        int prev_min = init_sec / 60;

        if(mins == prev_min) {
                invalidate();
                closeScreen();
        } else if(is_member && (mins > prev_min)) {
                seconds = (mins * 60) - init_sec;
                init_sec = mins * 60;
                remainingcheck(false);
        }
}

void KioskLock::lockupdate()
{
        if(!locked) {
                update(seconds / 60);
                closeScreen();
        }
}

void KioskLock::countdown()
{
        seconds--;
        remainingcheck(false, true);

        if(seconds == 0)
                lockinvalidate();
        else if(seconds == warning_init - 1) {
                timerbox->setBackgroundColor(RGB(255,128,0));
                timerbox->repaint();
                remainingcheck(true);
        } else if(seconds == warning_final - 1) {
                timerbox->startFlasher();
                remainingcheck(true);
        }
}

// update the acount on the server and lock the screen.
void KioskLock::updateLogoff()
{
	LogoutDialog l(this, "LogOut", true, Qt::WX11BypassWM);

	QDesktopWidget *d = QApplication::desktop();
        int x = d->width();
        int y = d->height();
        l.move((x/2) - (l.width() /2), 
	     (y/2) - l.height() );
	
	l.raise();
	if(l.exec() == QDialog::Rejected )
                return;        
        QApplication::flushX();
        if(seconds >= 60)
                lockupdate();
        else
                lockinvalidate();
}

// message to the user
void KioskLock::remainingcheck(bool warn, bool timerdisplay)
{
        unsigned int minutes = seconds / 60;
        unsigned int hhour = minutes / 60;
        unsigned int mmins = (minutes - (hhour * 60)) + 1;
        QString hs, ms;

        if(mmins == 60) {
                mmins = 0;
                hhour += 1;
        }
        // plural/singular thingy
        if(hhour > 1)
                hs = tr(" hours ");
        else
                hs = tr(" hour ");

        if(mmins > 1)
                ms = tr(" minutes ");
        else
                ms = tr(" minute ");

        QString message;
        if(hhour > 0)
                message = tr("You have %1 %2 and %3 %4 left")
                          .arg(hhour)
                          .arg(hs)
                          .arg(mmins)
                          .arg(ms);
        else
                message = tr("You have %1 %2 left")
                          .arg(mmins)
                          .arg(ms);

        if(warn) {
                QString	ws = tr("Please save your work to CD or "
                                "floppy disk and remove your files "
                                "from the computer for privacy!");
                infobox->setWarning(ws);
        }

        QString mstr;
        if(mmins < 10)
                mstr = QString::number(0) + QString::number(mmins);
        else
                mstr = QString::number(mmins);

        QString msg;
        if(hhour < 10)
                msg = QString::number(0) + QString::number(hhour) + QString(":") + mstr;
        else
                msg = QString::number(hhour) + QString(":") + mstr;

        if(timerdisplay) {
                timerbox->setText(msg);
                panel->setTimeDisplay(msg);
                emit timerUpdated(msg);
		timer_counter = msg;
        } else
                infobox->setInfoText(message);
}

void KioskLock::savestate()
{
        QFile file( ".kioskdata" );

        if(file.open( IO_WriteOnly )) {
                QDataStream stream( &file );
                stream << serial;   // serialize a string
                stream << key;
                stream << (Q_INT32)seconds;
                stream << (Q_INT32) is_member;
                file.close();
        } else
                QMessageBox::critical(this, "Error","Error saving user data.\
                                      Check file permissions!");
}

void KioskLock::reloadstate()
{
        QFile file( ".kioskdata" );

        if(file.open( IO_ReadOnly ) ) {
                QDataStream stream( &file );  // read the data serialized from the file
                Q_INT32 _lsecs; 
                Q_INT32 _lmember; 
                
                stream >> serial >> key >> _lsecs >> _lmember; 
                seconds = (int) _lsecs; 
                is_member = (bool) _lmember; 
                
                if(seconds > 0)  // should be > 60
                        update(seconds / 60);

                file.close();
        }

        emit memberStatus(is_member);
}

void KioskLock::ErrorDialog( QWidget*, const char* s)
{
        //   QMessageBox::warning(p, tr("Error"), s,
        //QMessageBox::Ok, QMessageBox::NoButton);
        msgBox(QMessageBox::Warning, s);
}

void KioskLock::InfoDialog( QWidget*, const char* s)
{
        //QMessageBox::information(p, tr("Information"), s,
        //QMessageBox::Ok, QMessageBox::NoButton);
        msgBox(QMessageBox::Information, s);
}

void KioskLock::hide()
{
        QWidget::hide();
        locked = false;

        // display the status
        if(seconds <= warning_init) {
                timerbox->setBackgroundColor(RGB(255,128,0));
                timerbox->repaint();
        }
        //	timerbox->show();
        getApplist();
        if(appsize > 0) {	// Enable Kiosk-mode
                panel->show();
                kioskmode = true;
                informUser("kiosk mode");
        } else 			// Disable Kiosk-mode
                kioskmode = false;
        
        infobox->show();
        sdlg->hide();
        ungrabInput();        

        emit sessionStart(kioskmode);
        if(is_emptypassword) {
                cpdialog->setNewAccount(true);
                cpdialog->show();
                cpdialog->setNewAccount(false);
        }        
}

void KioskLock::show()
{
        if(!grabInput()) {
                msgBox(QMessageBox::Warning, tr("Can't grab mouse or keyboad"));
                return;
        }
        QWidget::show();
        QApplication::flushX();
        setCursor( blankCursor );

        locked = true;
        timerbox->hide();
	panel->hide();
        infobox->hide();
        sdlg->setProgress(0);

        fakeFocusIn(sdlg->winId());
        sdlg->show();
        QTimer::singleShot(1000, this, SLOT(focusDialog()) );
        // client alive update
        updatetm.start(checkup_interval * 1000);
	
        panel->killApps();
}

void KioskLock::setServer(const QString &host, int port)
{
        serv_ip = host.ascii();
        serv_port = port;
}

void KioskLock::connectProgress(int c)
{
        emit connectionProgress(c);
}

void KioskLock::setWarnings(int init_warn, int final_warn)
{
        warning_init = init_warn;
        warning_final = final_warn;
}

KioskLock::~KioskLock() {}

void KioskLock::setClientInfo(const QString &station)
{
        _station = station;
}

void KioskLock::setClientPort(int port)
{
        client_port = port;
}

void KioskLock::resetTimeout()
{
        timeoutval = timeout_value;
}

void KioskLock::informUser(const char* msg, int duration)
{
        infobox->setInfoText(msg, duration);
}

void KioskLock::focusDialog()
{
        fakeFocusIn(sdlg->winId());
}

void KioskLock::setTaskbarWidget(QWidget* w)
{
        panel->setTaskbarWidget(w);
}

QString KioskLock::backgroundUrl() const
{
	return bgurl;
}

QString KioskLock::timerDisplay() const
{
	return timer_counter;
}

//---------------------------------------------------------------------------
// X11 specifics
//---------------------------------------------------------------------------
//---------------------------------------------------------------------------
//
// Grab the keyboard. Returns true on success
//
bool KioskLock::grabKeyboard()
{
        int rv = XGrabKeyboard( qt_xdisplay(), QApplication::desktop()->winId(),
                                True, GrabModeAsync, GrabModeAsync, CurrentTime );

        return (rv == GrabSuccess);
}

#define GRABEVENTS ButtonPressMask | ButtonReleaseMask | PointerMotionMask | \
		   EnterWindowMask | LeaveWindowMask

//---------------------------------------------------------------------------
//
// Grab the mouse.  Returns true on success
//
bool KioskLock::grabMouse()
{
        int rv = XGrabPointer( qt_xdisplay(), QApplication::desktop()->winId(),
                               True, GRABEVENTS, GrabModeAsync, GrabModeAsync, None,
                               arrowCursor.handle(), CurrentTime );
        
        return (rv == GrabSuccess);
}

//---------------------------------------------------------------------------
//
// Grab keyboard and mouse.  Returns true on success.
//
bool KioskLock::grabInput()
{
        XSync(qt_xdisplay(), False);
        
        if (!grabKeyboard())
        {
                sleep(1);
                if (!grabKeyboard())
                {
                        return false;
                }
        }
        
        if (!grabMouse())
        {
                sleep(1);
                if (!grabMouse())
                {
                        XUngrabKeyboard(qt_xdisplay(), CurrentTime);
                        return false;
                }
        }
        
        return true;
}

//---------------------------------------------------------------------------
//
// Release mouse and keyboard grab.
//
void KioskLock::ungrabInput()
{
    XUngrabKeyboard(qt_xdisplay(), CurrentTime);
    XUngrabPointer(qt_xdisplay(), CurrentTime);
}

//---------------------------------------------------------------------------
//
// X11 Event.
//
bool KioskLock::x11Event(XEvent *event)
{
        switch (event->type)
        {
                case FocusOut:
                        if(!event->xfocus.send_event && isVisible()) {
                                grabInput();
                                fakeFocusIn(sdlg->winId());
                        }
                        break;
                case VisibilityNotify:
                        if( event->xvisibility.window == winId()) { 
                                if (event->xvisibility.state != VisibilityUnobscured)
                                        stayOnTop();
                        }
                        break;
                        
                case ConfigureNotify: 
                        if(event->xconfigure.event == qt_xrootwin())
                                stayOnTop();
                        break;
                case MapNotify: 
                        if( event->xmap.event == qt_xrootwin())
                                stayOnTop();
                        break;
        }
        if(sdlg->isVisible() && ( event->type == KeyPress ||
                                  event->type == KeyRelease) 
           && event->xkey.window != sdlg->winId()) {
                XEvent ev2 = *event;
                ev2.xkey.window = ev2.xkey.subwindow = sdlg->winId();
                qApp->x11ProcessEvent( &ev2 );
                return true;
        }        
        return false;
}

void KioskLock::stayOnTop()
{
        if(sdlg->isVisible ()) 
                XRaiseWindow( qt_xdisplay(), sdlg->winId());
        else                
                XRaiseWindow( qt_xdisplay(), winId());
}

void KioskLock::fakeFocusIn( WId window )
{
    // We have keyboard grab, so this application will
    // get keyboard events even without having focus.
    // Fake FocusIn to make Qt realize it has the active
    // window, so that it will correctly show cursor in the dialog.
    XEvent ev;
    memset(&ev, 0, sizeof(ev));
    ev.xfocus.display = qt_xdisplay();
    ev.xfocus.type = FocusIn;
    ev.xfocus.window = window;
    ev.xfocus.mode = NotifyNormal;
    ev.xfocus.detail = NotifyAncestor;
    XSendEvent( qt_xdisplay(), window, False, NoEventMask, &ev );
}

void KioskLock::msgBox( QMessageBox::Icon type, const QString &txt )
{
    QDialog box( 0, "messagebox", true, WX11BypassWM );
    QFrame *winFrame = new QFrame( &box );
    winFrame->setFrameStyle( QFrame::WinPanel | QFrame::Raised );
    winFrame->setLineWidth( 2 );
    QLabel *label1 = new QLabel( winFrame );
    label1->setPixmap( QMessageBox::standardIcon( type ) );
    QLabel *label2 = new QLabel( txt, winFrame );
    QPushButton *button = new QPushButton( tr("Ok"), winFrame );
    button->setDefault( true );
    button->setSizePolicy( QSizePolicy( QSizePolicy::Preferred, QSizePolicy::Preferred ) );
    connect( button, SIGNAL( clicked() ), &box, SLOT( accept() ) );

    QVBoxLayout *vbox = new QVBoxLayout( &box );
    vbox->addWidget( winFrame );
    QGridLayout *grid = new QGridLayout( winFrame, 2, 2, 10 );
    grid->addWidget( label1, 0, 0, Qt::AlignCenter );
    grid->addWidget( label2, 0, 1, Qt::AlignCenter );
    grid->addMultiCellWidget( button, 1,1, 0,1, Qt::AlignCenter );

    fakeFocusIn(box.winId());
    box.exec();
    if(sdlg->isVisible())
            fakeFocusIn(sdlg->winId());
}
