/****************************************************************************
** ui.h extension file, included from the uic-generated form implementation.
**
** If you wish to add, delete or rename functions or slots use
** Qt Designer which will update this file, preserving your code. Create an
** init() function in place of a constructor, and a destroy() function in
** place of a destructor.
*****************************************************************************/
#include <qmessagebox.h>
#include <qtextstream.h>
#include <qfile.h>

#include <klocale.h>
#include <kstandarddirs.h>
#include <kconfig.h>
#include <kmessagebox.h>
#include <kprocess.h>
// POSIX
#include <sys/types.h>
#include <pwd.h>

void ClientSetup::help()
{
    QString helpmsg = tr2i18n(  "This is the setup tool for the OpenKiosk X11/KDE client.\n"	
				"In here, you can setup the location of the NodeView server and\n"
				"other client settings.\n\n" 
				"Specify the address of the NodeView server by filling up the values\n"
				"in the server host and port.\n\n"
				"The unique station name identifies this client to the NodeView server.\n"
				"It must match the station name specified for this particular client\n"
				"in the NodeView workstation configuration page.\n\n"
				"The OpenKiosk client should be installed and run in a separate\n"
				"non-admin UNIX user account specified in the UNIX account field.\n"
				"To automatically login using KDE auto-login under that UNIX user \n"
				"account, check the auto-login option. This option works on local\n"
				"X displays only.\n"
				"(Note: This setting is a security hole. You should only check this option\n"
				"on machines designated for kiosk purposes only. Otherwise, leave\n"
				"it unchecked to manually login on the UNIX account where the\n"
				"client is installed).\n\n"
				"To prevent people from removing the kiosk applet from the KDE panel,\n"
				"you should check the lock down applet option. Uncheck this option \n"
				"when uninstalling the client.\n\n"
				"If you want to change the default login and password label text in the\n"
				"login dialog, specify the new names in the login and password labels.\n"
				"The password entry can also be disabled if you don't\n"
				"want membership access. See OpenKiosk documentation for more details."
				);            
    QMessageBox::information(this, tr2i18n("OpenKiosk client setup help"), helpmsg,
			     QMessageBox::Ok, QMessageBox::NoButton);
}

void ClientSetup::save()
{    
    if(disable->isChecked())
	config->writeEntry("host", "none");
    else
	config->writeEntry("host", host->text());	
    config->writeEntry("port", port->text());
    config->writeEntry("station", station->text());
    config->writeEntry("user", user->text());
    config->writeEntry("autologin",autologin->isChecked());
    config->writeEntry("lockapplet",lockapplet->isChecked());
    config->writeEntry("loginlabel", loginlabel->text());
    config->writeEntry("passlabel", passlabel->text());
    config->writeEntry("enablepass", enablepass->isChecked());
    config->sync();
    
    // auto-login. TODO: remote displays
    KConfig kc(::locate("config", "kdm/kdmrc"));
    kc.setGroup("X-:0-Core");
    kc.writeEntry("AutoLoginUser", user->text());	
    autologin->isChecked() ? kc.writeEntry("AutoLoginEnable", true) :
	kc.writeEntry("AutoLoginEnable", false);
    kc.sync();
    // POSIX specific
    struct passwd* root = getpwuid(0);
    struct passwd* userp = getpwnam(user->text());
    QString rhome;
    QString userhome;
    if(root) rhome = root->pw_dir;
    if(userp) 
	userhome = userp->pw_dir;
    else
	KMessageBox::sorry(this, tr2i18n("Specified UNIX user does not exist. Client will not be "
					  "able to auto-login and locking down panel applet will "
					  "not be possible."));
    
    //applet lockdown
    if(lockapplet->isChecked()) {	
	QString globconfig= ::locate("config", "kioskclientrc").remove("kioskclientrc");
	// assume we're running as root
    // WARNING! This doesn't work on multi-user systems. Ideally, only lock down user-specific
    //          kickerrc. But then, wise user may manually edit kickerrc and remove the applet.
    //          Prevent access to all text editing applications?
	
	QString krloc = ::locateLocal("config", "kickerrc");
	krloc.replace(rhome, userhome);
	
	// Force KDE to trust us
	KConfig kic(krloc); 
	kic.setGroup("General");
	kic.writeEntry("UntrustedApplets","");
	kic.sync();

	QFile kickerrc(krloc);
	if ( kickerrc.open( IO_ReadOnly ) ) {
	    QTextStream stream( &kickerrc );
	    QString cs = stream.read();
	    QFile file( globconfig + "kickerrc" );
	    if( file.open( IO_WriteOnly) ) {
		QTextStream stream(&file  );
		stream << "[$i]\n";
		stream << cs;
		file.close();
	    }	
	    kickerrc.close();
	}	
    } else {
	KProcess p;
	p << "rm";
	p << ::locate("config", "kickerrc");
	p.start();
    }
    KProcess kproc;
    kproc.setUseShell(true);
    kproc << "killall" << "-9 kiosklock";
    kproc.start();

    // Can't use standard DCOP client classes because app won't authenticate 
    // when run as another user. Don't know yet how to specify another user 
    KProcess mdcop;
    mdcop.setUseShell(true);
    mdcop << KStandardDirs::findExe("dcop");
    mdcop << "--user" << user->text() << "kicker kicker restart";
    mdcop.start(KProcess::Block);
}

void ClientSetup::init()
{	
	
    config = new KConfig(::locate("config", "kioskclientrc"));
    config->setGroup("kioskclient");
    host->setText(config->readEntry("host"));
    port->setText(config->readEntry("port","10018"));
    station->setText(config->readEntry("station"));
    user->setText(config->readEntry("user"));
    autologin->setChecked(config->readBoolEntry("autologin",true));
    lockapplet->setChecked(config->readBoolEntry("lockapplet",true));
    loginlabel->setText(config->readEntry("loginlabel", "Username:"));
    passlabel->setText(config->readEntry("passlabel","Password:"));
    enablepass->setChecked(config->readBoolEntry("enablepass",true));  
}
    
void ClientSetup::destroy()
{
    delete config;
}
    
