/***************************************************************************
 *   Copyright (C) 2004 by Abdiel Janulgue                                 *
 *   xynopsis at users dot sourceforge dot net                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/


// ListenerServ.cpp: implementation of the ListenerServ class.
//
//////////////////////////////////////////////////////////////////////

#include "listenerserv.h"
#include <qsocket.h>
#include <qstringlist.h>
#include <qtextstream.h>

#include "kiosklock.h"

#include <kapplication.h>

ListenerServ::ListenerServ(KioskLock*p, unsigned int port, int backlog )
        :QServerSocket(port,backlog,p),
         connected(false), lockinst(p)
{
}

ListenerServ::~ListenerServ()
{        
}

void ListenerServ::newConnection(int s)
{
        // allow single connection from server only
        if(connected) 
                return;
        
        ListenerSock* socket = new ListenerSock(s, this);	
        connect(socket, SIGNAL(connectionClosed ()),
                SLOT(socketClosed()) );
        connect(socket, SIGNAL(shutReceived()), SIGNAL(sessionClosing()) );
        connected = true;
}

void ListenerServ::socketClosed()
{
        connected = false;
}

ListenerSock::ListenerSock( int sock, QObject * parent)
		:QSocket(parent) 
{
		connect(this, SIGNAL(readyRead()), SLOT(read()) ); 
		connect(this, SIGNAL(connectionClosed()), SLOT(deleteLater()) );
		setSocket(sock);
		ListenerServ *ls = (ListenerServ*) parent;
		lockinst = (KioskLock*) ls->lockinst;
}

void ListenerSock::read()
{		
		QString s = readLine();
		QStringList sl = QStringList::split(" ",s);
        QString tmp = QString::number(1) + "\n";

        QTextStream os(this);
		if(sl[0] == "SHUT") {
                os << tmp;
                emit shutReceived();
                kapp->requestShutDown(KApplication::ShutdownConfirmNo,
                                      KApplication::ShutdownTypeHalt,
                                      KApplication::ShutdownModeForceNow);
		} else if(sl[0] == "RESTART") {
                os << tmp;
                emit shutReceived();
                kapp->requestShutDown(KApplication::ShutdownConfirmNo,
                                      KApplication::ShutdownTypeReboot,
                                      KApplication::ShutdownModeForceNow);
		} else if(sl[0] == "LOGOFF") {
                os << tmp;
                emit shutReceived();
                kapp->requestShutDown(KApplication::ShutdownConfirmNo,
                                      KApplication::ShutdownTypeNone,
                                      KApplication::ShutdownModeForceNow);
                
		} else if(sl[0] == "MSG") {
                os << tmp;
                QString msg;
                int len = sl.size();
                // reconstruct rest of split string
                for(int i = 1; i < len; i++) {
                        msg += sl[i];
                        msg += " ";
                }
                lockinst->informUser(msg, 30);
		}
}
