/***************************************************************************
 *   Copyright (C) 2005 by Abdiel Janulgue                                 *
 *   xynopsis@users.sourceforge.net                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <kapplication.h>
#include <klocale.h>
#include <kcmdlineargs.h>
#include <kstandarddirs.h>
#include <kiconloader.h>
#include <kconfig.h>
#include <kprocess.h>
#include <dcopref.h>
#include <kaboutdata.h>
#include <kaboutapplication.h>
#include <khtml_part.h>
#include <khtmlview.h>

#include <qpopupmenu.h>
#include <qframe.h>

#include "kiosklock.h"
#include "kioskhandler.h"
#include "taskbar.h"
#include "listenerserv.h"

static KCmdLineOptions options[] =
{
    { "server <argument>", I18N_NOOP( "NodeView server host" ), 0 },
    { "port <argument>", I18N_NOOP( "NodeView server port" ), 0 },
    { "localname <argument>", I18N_NOOP( "This station's unique registered name" ), 
      0 },
    KCmdLineLastOption
};

int main( int argc, char **argv )
{              
        KCmdLineArgs::init( argc, argv,"kiosklock",
			     I18N_NOOP("KioskLock"),
			     I18N_NOOP("OpenKiosk Client"), 
			    "2.0.4" );
	KCmdLineArgs::addCmdLineOptions(options);
        KApplication a;
       
	KioskLock kl(0,"KioskLock");
	KioskHandler kh(&kl, "KioskHandler");
	
        a.connect(&kl, SIGNAL(sessionStart(bool)), &kh, 
		  SLOT(kioskStart(bool)) );
        a.connect(&kl, SIGNAL(sessionEnd(bool)), &kh, 
		  SLOT(kioskEnd(bool)) );
	
        KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
	if( !args->isSet("server"))
		args->usage(QString("NodeView server host needed"));
	if(!args->isSet("localname"))
		args->usage(QString("This station's unique registered name needed"));
	if(!args->isSet("port"))
		args->usage(QString("NodeView server port needed"));

	QString host = args->getOption("server");
	QString name = args->getOption("localname");
	int port = args->getOption("port").toInt();

	kl.setServer(host, port);
	kl.setClientInfo(name);
	kl.reloadstate();

	ListenerServ l(&kl, 0);
        if(l.ok()) kl.setClientPort(l.port());
        
        a.setMainWidget( &kl);

	// check if nodeview is available initially
	if(!kl.isNodeviewUp()) 
		kl.InfoDialog(0, "There is a problem connecting to "
			      "NodeView. The client will\n"
			      "be disabled so you can reconfigure it. "
			      "Run kiosksetup as \n"
			      "root if you believe you made "
			      "an error in the\n"
			      "connection settings and restart the "
			      "client.");
	else
		kl.show();
        return a.exec();
}

KioskHandler::KioskHandler(KioskLock* parent, const char* name)
	:QObject(parent, name), ab(0)
{
	pop = new QPopupMenu(parent);
	kl = parent;
        pop->insertItem(::SmallIcon("clock"),
                        tr2i18n("&Check Remaining Time"), kl,
                        SLOT(checkTime()) );
    	pop->insertSeparator ();
        pop->insertItem(::SmallIcon("exit"),
                        tr2i18n("&Log off or pause usage"), kl,
                        SLOT(updateLogoff()));
        pop->insertItem(::SmallIcon("reload"),
                        tr2i18n("&Reload/extend account"), kl,
                        SLOT(reload()));
        pop->insertItem(::SmallIcon("password"),
                        tr2i18n("Change my &Password"),kl,
                        SLOT(changePassword()));
        pop->insertSeparator();
        pop->insertItem(::SmallIcon("configure"),
                        tr2i18n("Confi&gure"), this,
                        SLOT(preferences()));
        pop->insertItem(tr2i18n("&About"), this,
                        SLOT(about()));
        taskbar = new TaskBar(pop, 0, "taskbar");
	kl->setTaskbarWidget(taskbar);
	
	bwidget = new QWidget(0, "Background", WX11BypassWM);
	bview = new KHTMLPart(bwidget);
	QDesktopWidget *d = QApplication::desktop();
        int x = d->width();
        int y = d->height();
	
	bwidget->move(0,0);
	bwidget->resize(x,y);
	bview->view()->setFrameShadow (QFrame::Plain);
	bview->view()->resize(x, y);
	bwidget->lower();
}

KioskHandler::~KioskHandler()
{
	delete bwidget;
}

void KioskHandler::disableDesktop()
{
	DCOPRef dsk("kdesktop", "MainApplication-Interface");
	if(!dsk.isNull()) 
		dsk.call("quit");
}

void KioskHandler::disableKicker()
{
        DCOPRef kicker("kicker", "MainApplication-Interface");
	if(!kicker.isNull()) 
		kicker.call("quit");
}

void KioskHandler::kioskStart(bool isKiosk)
{
        KConfig glob("kdeglobals");

	if(isKiosk) {
		// enable KDE restrictions
                glob.setGroup("KDE Action Restrictions");
                glob.writeEntry("shell_access", false);
                glob.writeEntry("run_command", false);
		
                // use our own task manager
                //taskbar->closeAllWindows();
                taskbar->show();
		
		// set background
		setBackgroundUrl(kl->backgroundUrl());

		QTimer *t = new QTimer( this );
		connect( t, SIGNAL(timeout()), SLOT(disableKicker()) );
		connect( t, SIGNAL(timeout()), SLOT(disableDesktop()) );
		t->start( 0, true );
	} else {
		glob.setGroup("KDE Action Restrictions");
		glob.writeEntry("shell_access", true);
		glob.writeEntry("run_command", true);
	}
	glob.sync();
}

void KioskHandler::kioskEnd(bool isKiosk)
{	
        KConfig glob("kdeglobals");
	glob.setGroup("KDE Action Restrictions");
	glob.writeEntry("shell_access", true);
	glob.writeEntry("run_command", true);
	glob.sync();

	if(isKiosk) {
		 KProcess p;
		 p << "kdesktop";
		 p.start(KProcess::DontCare);

		 KProcess p2;
		 p2 << "kicker";
		 p2.start(KProcess::DontCare);

		 kapp->quit();
	}
}

void KioskHandler::preferences()
{
	KProcess p;
        p << "kdesu";
        p << "kiosksetup";
        p.start(KProcess::DontCare);
}

void KioskHandler::about()
{
        if(!ab) {
                ab = new KAboutData("opkdekiosk", I18N_NOOP("OpenKDEKiosk"), 
                                    "2.0.4", I18N_NOOP("The KDE client of "
						      "OpenKiosk for "
                                                      "UNIX, Linux, and "
						      "BSD on X11"),
                                    KAboutData::License_GPL, 
				    "(c) 2005 by xynopsis",
				    I18N_NOOP("Hinimo sa pinoy!"),
                                    "http://openkiosk.sourceforge.net");
                ab->addAuthor("Abdiel B. Janulgue",
                              "http://openkiosk.sourceforge.net",
                              "xynopsis@users.sourceforge.net");
        }

        KAboutApplication dialog(ab);
        dialog.exec();
}

void KioskHandler::setBackgroundUrl(const QString& u)
{
	KURL url = u;
	QString defurl = QString("file:") + 
		::locate("data","openkiosk/defaultbg/index.html");

	if(!url.isValid())
		url = defurl;
	
	bview->openURL(url);
	bwidget->show();	
}
