/***************************************************************************
 *   Copyright (C) 2004 by Abdiel Janulgue                                 *
 *   xynopsis at users dot sourceforge dot net                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

// OpenKiosk Win32 client - Win32Lock

// 220 pixels - width of pass and login entry
#include "serialdialog.h"
#include "kiosklock.h"
#include <qpixmap.h>
#include <qmessagebox.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qprogressbar.h>
#include <qapplication.h>
#include <qevent.h>
#include <qlayout.h> 

#include <kstandarddirs.h>

// safe user name and password entry widget
class AccessEdit: public QLineEdit
{
public:
        AccessEdit(QWidget* parent, const char* name =0)
                :QLineEdit(parent, name)
                {}

protected:
        void contextMenuEvent(QContextMenuEvent*)
        {
                // Ignore directly
                return;
        }
};

SerialDialog::SerialDialog(KioskLock *lock, int WFlags)
		:LoginDialog(lock, "SerialDialog", false, WFlags),
         klock(lock)
{
        accessEdit = new AccessEdit(frame7, "access");
        accessEdit->setMinimumSize( QSize( 220,0 ) );
        passEdit = new AccessEdit(frame7, "pass");
        passEdit->setMinimumSize( QSize( 220, 0) );
        passEdit->setEchoMode(QLineEdit::Password);
        accessLayout->insertWidget(2, accessEdit);
        passLayout->insertWidget(2, passEdit);
        
        connect(okb, SIGNAL(clicked()), SLOT(gettime()));
        connect(helpb, SIGNAL(clicked()), SLOT(help()));
        setBmpLogo(locate("data", "openkiosk/logo.bmp"));
}

SerialDialog::~SerialDialog()	{ }

void SerialDialog::hide()
{
	if(klock->isVisible())
		return;
	else
		QDialog::hide();
}

void SerialDialog::show()
{
        QDialog::show();
//         raise();
//         setActiveWindow();
        QApplication::flushX();
}

void SerialDialog::setInfoText(const QString &name, const QString &pass, 
							   const QString & hlpmsg, bool enablepass)
{
	accessLabel->setText(name);
	passLabel->setText(pass);
	helptext = hlpmsg;
	passEdit->setEnabled(enablepass);
}

void SerialDialog::setBmpLogo(const char* filename)
{
        logoPix->setPixmap(QPixmap(filename));
}

void SerialDialog::help()
{
        klock->InfoDialog(this, helptext);
}

void SerialDialog::gettime()
{
        QString str = accessEdit->text();
        QString strpass = passEdit->text();
        int seconds = klock->gettime(str, strpass);
        klock->openScreen(seconds * 60, str, strpass);
        progress->setProgress(0);
        accessEdit->setText("");
        passEdit->setText("");
}

void SerialDialog::setProgress(int p)
{
        progress->setProgress(p);
}

