#include "taskbar.h"
#include <qapplication.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qdockarea.h>
#include <qhbox.h>
#include <qpushbutton.h>
#include <qmessagebox.h>
#include <qtooltip.h>
#include <qpopupmenu.h>
#include <qcursor.h>

#include <kwin.h>
#include <kwinmodule.h>
#include <klocale.h>
#include <kiconloader.h>
#include <netwm.h>

Task::Task(TaskBar* parent, WId id, const char* name)
        :QToolButton(parent->tb, name),
         kwm(parent->kwm), taskid(id), taskbar(parent)
{
        taskbar->tasks++;
        setToggleButton(true);
        setUsesTextLabel ( true);
        setTextPosition ( BesideIcon );

        pop = new QPopupMenu(this);
        pop->insertItem(::SmallIcon("fileclose"),
                        tr2i18n("&Close"), this,
                        SLOT(close()) );
        
        connect(this, SIGNAL(clicked()), SLOT(activate()) );
        connect(this, SIGNAL(clicked()), SLOT(requestActive()) );
        sep = new QFrame(parent->tb);
        sep->setFrameShape(QFrame::VLine);
        sep->setFrameShadow(QFrame::Sunken);
}

Task::~Task()
{
        delete sep;
        taskbar->tasks--;
        if(taskbar->tasks == 0)
                taskbar->activeTask = 0;
}

void Task::close()
{
        NETRootInfo root( qt_xdisplay(), NET::CloseWindow);
        root.closeWindowRequest(taskid);
}

void Task::mousePressEvent(QMouseEvent*e)
{
        QToolButton::mousePressEvent(e);
        if(e->button() == RightButton)
                pop->exec(QCursor::pos());
}

void Task::requestActive()
{
        KWin::WindowInfo inf = KWin::windowInfo(taskid);

        if(kwm->activeWindow() == taskid)
                KWin::iconifyWindow(taskid);
        else // requires kde > 3.2
                KWin::activateWindow(taskid);

        if(inf.isMinimized())
                KWin::deIconifyWindow(taskid);
}

void Task::activate()
{
        setOn(true);
        if(taskbar->activeTask && (taskbar->activeTask != this))
                taskbar->activeTask->setOn(false);
        taskbar->activeTask = this;
}

TaskBar::TaskBar(QPopupMenu* kioskmenu, QWidget* parent, const char* name)
        :QHBox(parent, name, WStyle_Customize |  
	       WType_TopLevel | WX11BypassWM),
         activeTask(0),tasks(0)
{
        setFrameStyle( QFrame::WinPanel | QFrame::Raised );
        da = new QDockArea(Qt::Horizontal,QDockArea::Normal, this);
        tb = new QToolBar(0, "Tb");
        tb->setFixedExtentHeight(25);
        tb->setMovingEnabled ( false);
        tb->setHorizontallyStretchable (false);
        
        da->moveDockWindow ( tb);
        mb = new QPushButton (tr2i18n("My account"), this);
        mb->setPopup(kioskmenu);
        kwm = new KWinModule(this);
        connect(kwm, SIGNAL(windowAdded(WId)), SLOT(addWindow(WId)) );
        connect(kwm, SIGNAL(windowRemoved(WId)), SLOT(removeWindow(WId)) );
        connect(kwm, SIGNAL(activeWindowChanged(WId)), SLOT(activateWindow(WId)) );
        
        QDesktopWidget *d = QApplication::desktop();
        int x = d->width();
        int y = d->height();
        move(0,y-30);
        resize(x,30);

        QValueList<WId> wlist  = kwm->windows();
        QValueList<WId>::iterator it;
        for ( it = wlist.begin(); it != wlist.end(); ++it )
                addWindow(*it);
}

void TaskBar::addWindow(WId id)
{        
	KWin::WindowInfo inf = KWin::windowInfo(id);
        QString caption = inf.visibleName();
        if(caption == "kicker")
                return;

        if(isVisible())         
                KWin::setStrut(id, 0, 0, 0, 30);
        
        QString vname = caption;
        if(vname.length() > 25) {
                vname.truncate(25);
                vname += "...";
        } else
                vname = vname.leftJustify(25);
        Task* t = new Task(this, id, vname);
        t->setTextLabel(vname, false);
        t->setIconSet(KWin::icon(id,16,16,true));
        QToolTip::add(t, caption);
        tasklist.push_back(t);        
}

void TaskBar::removeWindow(WId id)
{       
        Task* del = 0;
        for(TaskList::iterator it = tasklist.begin(); it != tasklist.end(); ++it)
                if((*it)->taskId() == id) {
                        del = *it;
                        break;
                }
        if(del) {
                tasklist.remove(del);
                del->deleteLater();
        }
        activeTask = 0;        
}

void TaskBar::activateWindow(WId id)
{        
        for(TaskList::iterator it = tasklist.begin(); it != tasklist.end(); ++it)
                if((*it)->taskId() == id) {
                        (*it)->activate();
                        break;
                }
}

void TaskBar::closeAllWindows()
{
        for(TaskList::iterator it = tasklist.begin(); it != tasklist.end(); ++it)
                (*it)->close();
}

TaskBar::~TaskBar()
{
        delete tb;
}
